@@ -1,12 +1,11 @@
-
 package Term::TablePrint;
 
 use warnings;
 use strict;
-use 5.008000;
+use 5.008003;
 no warnings 'utf8';
 
-our $VERSION = '0.018_01';
+our $VERSION = '0.026_01';
 use Exporter 'import';
 our @EXPORT_OK = qw( print_table );
 
@@ -14,13 +13,12 @@ use Carp         qw( carp croak );
 use List::Util   qw( sum );
 use Scalar::Util qw( looks_like_number );
 
-use Term::Choose       qw( choose );
-use Term::Choose::Util qw( term_size insert_sep unicode_sprintf );
-use Term::ProgressBar  qw();
-use Text::LineFold     qw();
-use Unicode::GCString  qw();
+use Term::Choose           qw( choose );
+use Term::Choose::LineFold qw( line_fold cut_to_printwidth print_columns );
+use Term::Choose::Util     qw( term_size insert_sep unicode_sprintf );
+use Term::ProgressBar      qw();
 
-sub CLEAR_SCREEN () { "\e[H\e[J" }
+use constant CLEAR_SCREEN => "\e[H\e[J";
 
 
 
@@ -39,27 +37,23 @@ sub new {
 
 sub __validate_options {
     my ( $self, $opt ) = @_;
-    if ( $opt->{_db_browser_mode} || $opt->{db_browser_mode} ) { ###
-        %$self = ( %$self, %$opt );
-        return;
-    }
     my $valid = {
-        progress_bar    => '[ 0-9 ]+',
         max_rows        => '[ 0-9 ]+',
         min_col_width   => '[ 0-9 ]+',
+        progress_bar    => '[ 0-9 ]+',
         tab_width       => '[ 0-9 ]+',
-        binary_filter   => '[ 0 1 ]',
         add_header      => '[ 0 1 ]',
+        binary_filter   => '[ 0 1 ]',
         keep_header     => '[ 0 1 ]',
-        table_expand    => '[ 0 1 ]',
         choose_columns  => '[ 0 1 2 ]',
+        table_expand    => '[ 0 1 2 ]',
         mouse           => '[ 0 1 2 3 4 ]',
         binary_string   => '',
+        prompt          => '',
         undef           => '',
         #thsd_sep       => '',
         #no_col         => '',
     };
-
     for my $key ( keys %$opt ) {
         if ( ! exists $valid->{$key} ) {
             carp "print_table: '$key' is not a valid option name.";
@@ -82,18 +76,19 @@ sub __validate_options {
 
 sub __set_defaults {
     my ( $self ) = @_;
-    $self->{progress_bar}   = 40000  if ! defined $self->{progress_bar};
-    $self->{max_rows}       = 50000  if ! defined $self->{max_rows};
-    $self->{tab_width}      = 2      if ! defined $self->{tab_width};
-    $self->{min_col_width}  = 30     if ! defined $self->{min_col_width};
-    $self->{table_expand}   = 1      if ! defined $self->{table_expand};
+    $self->{add_header}     = 0      if ! defined $self->{add_header};
     $self->{binary_filter}  = 0      if ! defined $self->{binary_filter};
-    $self->{undef}          = ''     if ! defined $self->{undef};
-    $self->{mouse}          = 0      if ! defined $self->{mouse};
     $self->{binary_string}  = 'BNRY' if ! defined $self->{binary_string};
     $self->{choose_columns} = 0      if ! defined $self->{choose_columns};
-    $self->{add_header}     = 0      if ! defined $self->{add_header};
     $self->{keep_header}    = 1      if ! defined $self->{keep_header};
+    $self->{max_rows}       = 50000  if ! defined $self->{max_rows};
+    $self->{min_col_width}  = 30     if ! defined $self->{min_col_width};
+    $self->{mouse}          = 0      if ! defined $self->{mouse};
+    $self->{progress_bar}   = 40000  if ! defined $self->{progress_bar};
+    $self->{prompt}         = ''     if ! defined $self->{prompt};
+    $self->{tab_width}      = 2      if ! defined $self->{tab_width};
+    $self->{table_expand}   = 1      if ! defined $self->{table_expand};
+    $self->{undef}          = ''     if ! defined $self->{undef};
     $self->{thsd_sep} = ',';
     $self->{no_col}   = 'col';
 }
@@ -105,11 +100,10 @@ sub __choose_columns_with_order {
     my $ok = '-ok-';
     my @pre = ( $ok );
     my $init_prompt = 'Columns: ';
-    my $gcs = Unicode::GCString->new( $init_prompt );
-    my $s_tab = $gcs->columns();
+    my $s_tab = print_columns( $init_prompt );
 
     while ( 1 ) {
-        my @chosen_cols = @$col_idxs ? map( $avail_cols->[$_], @$col_idxs ) : '*';
+        my @chosen_cols = @$col_idxs ?  @{$avail_cols}[@$col_idxs] : '*';
         my $prompt = $init_prompt . join ', ', @chosen_cols;
         my $choices = [ @pre, @$avail_cols ];
         # Choose
@@ -118,7 +112,7 @@ sub __choose_columns_with_order {
             { prompt => $prompt, lf => [ 0, $s_tab ], clear_screen => 1,
               no_spacebar => [ 0 .. $#pre ], index => 1, mouse => $self->{mouse} }
         );
-        if ( ! @idx || ! defined $choices->[$idx[0]] ) {
+        if ( ! @idx || ! defined $choices->[$idx[0]] ) { #
             if ( @$col_idxs ) {
                 $col_idxs = [];
                 next;
@@ -148,7 +142,9 @@ sub __choose_columns_simple {
         $choices,
         { prompt => 'Choose: ', no_spacebar => [ 0 .. $#pre ], index => 1, mouse => $self->{mouse} }
     );
-    return if ! @idx;
+    if ( ! @idx ) {
+        return;
+    }
     if ( $choices->[$idx[0]] eq $all ) {
         return [];
     }
@@ -162,64 +158,41 @@ sub print_table {
     }
     my $self = shift;
     my ( $table_ref, $opt ) = @_;
-    my  $a_ref;
-    if ( $self->{_db_browser_mode} || $self->{db_browser_mode} ) { ###
-        $self->__set_defaults();
-        $a_ref = $table_ref;
+    croak "print_table: called with " . @_ . " arguments - 1 or 2 arguments expected." if @_ < 1 || @_ > 2;
+    croak "print_table: Required an ARRAY reference as the first argument."            if ref $table_ref  ne 'ARRAY';
+    croak "print_table: Empty table without header row!"                               if ! @$table_ref;
+    if ( defined $opt ) {
+        croak "print_table: The (optional) second argument is not a HASH reference."   if ref $opt ne 'HASH';
+        $self->{backup_opt} = { map{ $_ => $self->{$_} } keys %$opt };
+        $self->__validate_options( $opt );
+    }
+    $self->__set_defaults();
+    if ( $self->{add_header} ) {
+        unshift @$table_ref, [ map { $_ . '_' . $self->{no_col} } 1 .. @{$table_ref->[0]} ];
+    }
+    my $last_row_idx = $self->{max_rows} && $self->{max_rows} < @$table_ref ? $self->{max_rows} : $#$table_ref;
+    my $col_idxs = [];
+    if ( $self->{choose_columns}  ) {
+        $col_idxs = $self->__choose_columns_simple( $table_ref->[0] )     if $self->{choose_columns} == 1;
+        $col_idxs = $self->__choose_columns_with_order( $table_ref->[0] ) if $self->{choose_columns} == 2;
+        return if ! defined $col_idxs;
+    }
+    my $a_ref = [];
+    if ( @$col_idxs ) {
+        $a_ref = [ map { [ @{$table_ref->[$_]}[@$col_idxs] ] } 0 .. $last_row_idx ];
     }
     else {
-        croak "print_table: called with " . @_ . " arguments - 1 or 2 arguments expected." if @_ < 1 || @_ > 2;
-        croak "print_table: Required an ARRAY reference as the first argument."            if ref $table_ref  ne 'ARRAY';
-        croak "print_table: Empty table without header row!"                               if ! @$table_ref;
-        if ( defined $opt ) {
-            croak "print_table: The (optional) second argument is not a HASH reference."   if ref $opt ne 'HASH';
-            $self->{backup_opt} = { map{ $_ => $self->{$_} } keys %$opt } if defined $opt;
-            $self->__validate_options( $opt );
-        }
-        $self->__set_defaults();
-        if ( $self->{add_header} ) {
-            unshift @$table_ref, [ map { $_ . '_' . $self->{no_col} } 1 .. @{$table_ref->[0]} ];
-        }
-        my $last_row_idx = $self->{max_rows} && $self->{max_rows} < @$table_ref ? $self->{max_rows} : $#$table_ref;
-        my @copy = ();
-        if ( $self->{choose_columns}  ) {
-            my $col_idxs;
-            $col_idxs = $self->__choose_columns_simple( $table_ref->[0] )     if $self->{choose_columns} == 1;
-            $col_idxs = $self->__choose_columns_with_order( $table_ref->[0] ) if $self->{choose_columns} == 2;
-            return if ! defined $col_idxs;
-            if ( @$col_idxs ) {
-                @copy = map { [ @{$table_ref->[$_]}[@$col_idxs] ] } 0 .. $last_row_idx;
-            }
-        }
-        if ( @copy ) {
-            $a_ref = \@copy;
-        }
-        else {
-            $a_ref = $table_ref;
+        $a_ref = $table_ref;
+        if ( $last_row_idx < $#$table_ref ) {
             $#$a_ref = $last_row_idx;
         }
     }
-    my $gcs_bnry = Unicode::GCString->new( $self->{binary_string} );
-    $self->{binary_length} = $gcs_bnry->columns;
+    $self->{binray_regexp} = qr/[\x00-\x08\x0B-\x0C\x0E-\x1F]/;
     if ( $self->{progress_bar} ) {
         print 'Computing: ...' . "\n";
         $self->{show_progress} = int @$a_ref * @{$a_ref->[0]} / $self->{progress_bar};
     }
     $self->__calc_col_width( $a_ref );
-
-    #########################
-    if ( defined wantarray ) {
-        if ( wantarray ) {
-            my @idxs = $self->__inner_print_tbl( $a_ref );
-            return @idxs;
-        }
-        else {
-            my $idx = $self->__inner_print_tbl( $a_ref );
-            return $idx;
-        }
-    }
-    #########################
-
     $self->__inner_print_tbl( $a_ref );
     if ( $self->{backup_opt} ) {
         my $backup_opt = delete $self->{backup_opt};
@@ -234,45 +207,19 @@ sub __inner_print_tbl {
     my ( $self, $a_ref ) = @_;
     my ( $term_width ) = term_size();
     my $width_cols = $self->__calc_avail_width( $a_ref, $term_width );
-    return if ! $width_cols;
     my ( $list, $len ) = $self->__trunk_col_to_avail_width( $a_ref, $width_cols );
     if ( $self->{max_rows} && @$list - 1 >= $self->{max_rows} ) {
-        my $reached_limit = 'REACHED LIMIT "MAX_ROWS": ' . insert_sep( $self->{max_rows}, $self->{thsd_sep} );
-        my $gcs1 = Unicode::GCString->new( $reached_limit );
-        if ( $gcs1->columns > $len ) {
-            $reached_limit = 'REACHED LIMIT';
-            my $gcs2 = Unicode::GCString->new( $reached_limit );
-            if ( $gcs2->columns > $len ) {
-                $reached_limit = '=LIMIT=';
-            }
+        my $limit = insert_sep( $self->{max_rows}, $self->{thsd_sep} );
+        my $reached_limit = 'REACHED LIMIT "MAX_ROWS": ' . $limit;
+        if ( print_columns( $reached_limit ) > $len ) {
+            $reached_limit = '=LIMIT= ' . $limit;
+            $reached_limit = cut_to_printwidth( $reached_limit, $len );
         }
         push @$list, unicode_sprintf( $reached_limit, $len, 0 );
     }
-
-    #########################
-    if ( defined wantarray ) {
-        my $prompt = 'Your choice:';
-        #shift @$list; # The index 0 is the index of the row after the header row.
-        if ( wantarray ) {
-            my @idxs = choose(
-                $list,
-                { prompt => $prompt, index => 1, ll => $len, layout => 3, clear_screen => 1, mouse => $self->{mouse} }
-            );
-            return @idxs;
-        }
-        else {
-            my $idx = choose(
-                $list,
-                { prompt => $prompt, index => 1, ll => $len, layout => 3, clear_screen => 1, mouse => $self->{mouse} }
-            );
-            return $idx;
-        }
-    }
-    #########################
-
-    #my $old_row = $self->{keep_header} ? @$list : 0;
     my $old_row = 0;
-    my $auto_jump = 1;
+    my $auto_jumped_to_first_row = 2;
+    my $expanded = 0;
     my ( $width ) = term_size();
     while ( 1 ) {
         if ( ( term_size() )[0] != $width ) {
@@ -280,153 +227,114 @@ sub __inner_print_tbl {
             $self->__inner_print_tbl( $a_ref );
             return;
         }
-        my $prompt = $self->{keep_header} ? shift @$list : '';
+        my $header = $self->{keep_header} ? shift @$list : '';
         if ( ! @$list ) {
-            push @$list, $prompt;
-            $prompt = '';
-            $old_row = 0;
+            $list = [ $header ];
+            $header = undef;
         }
+        my $prompt;
+        if ( length $self->{prompt} ) {
+            $prompt = $self->{prompt} . "\n" . $header;
+        }
+        else {
+            $prompt = $header;
+        }
+        # Choose
         my $row = choose(
             $list,
             { prompt => $prompt, index => 1, default => $old_row, ll => $len, layout => 3,
               clear_screen => 1, mouse => $self->{mouse} }
         );
-        unshift @$list, $prompt if $self->{keep_header};
-        return if ! defined $row;
-        #if ( ! $self->{table_expand} ) {
-        #    return if $row == 0;
-        #    next;
-        #}
-        #if ( $old_row == $row ) {
-        #    return if $row == 0;
-        #    $old_row = 0;
-        #    next;
-        #}
+        if ( ! defined $row ) {
+            return;
+        }
+        if ( $self->{keep_header} && defined $header ) {
+            unshift @$list, $header;
+        }
         if ( ! $self->{table_expand} ) {
             return if $row == 0;
-            next;
         }
-        if ( $old_row == $row && ! $self->{keep_header} ) {
-            return if $row == 0;
-            $old_row = 0;
-            next;
+        else {
+            if ( $old_row == $row ) {
+                if ( $row == 0 ) {
+                    if ( ! $self->{keep_header} ) {
+                        return;
+                    }
+                    elsif ( $self->{table_expand} == 1 ) {
+                        return if $expanded;
+                        return if $auto_jumped_to_first_row == 1;
+                    }
+                    elsif ( $self->{table_expand} == 2 ) {
+                        return if $expanded;
+                    }
+                    $auto_jumped_to_first_row = 0;
+                }
+                else {
+                    $old_row = 0;
+                    $auto_jumped_to_first_row = 1;
+                    $expanded = 0;
+                    next;
+                }
+            }
+            $old_row = $row;
+            $row++ if $self->{keep_header};
+            $expanded = 1;
+            $self->__print_single_row( $a_ref, $row, $self->{longest_col_name} + 1 );
         }
-        if ( $old_row == $row && ! $auto_jump ) {
-            return if $row == 0;
-            $old_row = 0;
-            $auto_jump = 1;
-            next;
-        }
-        $auto_jump = 0;
-        $old_row = $row;
-        $row++ if $prompt;
-
-        my $row_data = $self->__single_row( $a_ref, $row, $self->{longest_col_name} + 1 );
-        choose(
-            $row_data,
-            { prompt => '', layout => 3, clear_screen => 1, mouse => $self->{mouse} }
-        );
     }
 }
 
 
-sub __single_row {
+sub __print_single_row {
     my ( $self, $a_ref, $row, $len_key ) = @_;
     my ( $term_width ) = term_size();
     $len_key = int( $term_width / 100 * 33 ) if $len_key > int( $term_width / 100 * 33 );
     my $separator = ' : ';
-    my $gcs_sep = Unicode::GCString->new( $separator );
-    my $len_sep = $gcs_sep->columns;
+    my $len_sep = print_columns( $separator );
     my $col_max = $term_width - ( $len_key + $len_sep + 1 );
-    my $line_fold = Text::LineFold->new(
-        Charset=> 'utf8',
-        OutputCharset => '_UNICODE_',
-        Urgent => 'FORCE' ,
-        ColMax => $col_max,
-    );
     my $row_data = [ ' Close with ENTER' ];
+
     for my $col ( 0 .. $#{$a_ref->[0]} ) {
         push @{$row_data}, ' ';
-        my $key = $a_ref->[0][$col];
+        my $key = cut_to_printwidth( $self->__sanitized_string( $a_ref->[0][$col] ), $len_key );
         my $sep = $separator;
         if ( ! defined $a_ref->[$row][$col] || $a_ref->[$row][$col] eq '' ) {
             push @{$row_data}, sprintf "%*.*s%*s%s", $len_key, $len_key, $key, $len_sep, $sep, '';
         }
         else {
-            my $text = $line_fold->fold( $a_ref->[$row][$col], 'PLAIN' );
-            for my $line ( split /\R+/, $text ) {
+            for my $line ( split /\n+/, line_fold( $a_ref->[$row][$col], $col_max, '', '' ) ) {
                 push @{$row_data}, sprintf "%*.*s%*s%s", $len_key, $len_key, $key, $len_sep, $sep, $line;
                 $key = '' if $key;
                 $sep = '' if $sep;
             }
         }
     }
-    return $row_data;
+    # Choose
+    choose(
+        $row_data,
+        { prompt => '', layout => 3, clear_screen => 1, mouse => $self->{mouse} }
+    );
 }
 
 
-sub __calc_col_width {
-    my ( $self, $a_ref ) = @_;
-    my $binray_regexp = qr/[\x00-\x08\x0B-\x0C\x0E-\x1F]/;
-    $self->{longest_col_name} = 0;
-    my $normal_row = 0;
-    $self->{width_cols} = [ ( 1 ) x @{$a_ref->[0]} ];
-    my @col_idx = ( 0 .. $#{$a_ref->[0]} );
-    my $show_progress = $self->{show_progress} >= 2 ? 1 : 0; #
-    my $total = $#{$a_ref};                   #
-    my $next_update = 0;                      #
-    my $c = 0;                                #
-    my $progress;                             #
-    if ( $show_progress ) {                   #
-        local $| = 1;                         #
-        print CLEAR_SCREEN;                   #
-        $progress = Term::ProgressBar->new( { #
-            name => 'Computing',              #
-            count => $total,                  #
-            remove => 1 } );                  #
-        $progress->minor( 0 );                #
-    }                                         #
-    for my $row ( @$a_ref ) {
-        for my $i ( @col_idx ) {
-            $row->[$i] = $self->{undef} if ! defined $row->[$i];
-            if ( ref $row->[$i] ) {
-                $row->[$i] = $self->__handle_reference( $row->[$i] );
-            }
-            my $width;
-            if ( $self->{binary_filter} && substr( $row->[$i], 0, 100 ) =~ $binray_regexp ) {
-                $row->[$i] = $self->{binary_string};
-                $width     = $self->{binary_length};
-            }
-            else {
-                $row->[$i] =~ s/^\p{Space}+//;
-                $row->[$i] =~ s/\p{Space}+\z//;
-                #$row->[$i] =~ s/(?<=\P{Space})\p{Space}+/ /g;
-                $row->[$i] =~ s/\p{Space}+/ /g;
-                $row->[$i] =~ s/\p{C}//g;
-                my $gcs = Unicode::GCString->new( $row->[$i] );
-                $width = $gcs->columns;
-            }
-            if ( $normal_row ) {
-                $self->{width_cols}[$i] = $width if $width > $self->{width_cols}[$i];
-                ++$self->{not_a_number}[$i] if $row->[$i] && ! looks_like_number $row->[$i];
-            }
-            else {
-                # column name
-                $self->{width_head}[$i] = $width;
-                $self->{longest_col_name} = $width if $width > $self->{longest_col_name};
-                $normal_row = 1 if $i == $#$row;
-            }
-        }
-        if ( $show_progress ) {                                              #
-            my $is_power = 0;                                                #
-            for ( my $i = 0; 2 ** $i <= $c; ++$i ) {                         #
-                $is_power = 1 if 2 ** $i == $c;                              #
-            }                                                                #
-            $next_update = $progress->update( $c ) if $c >= $next_update;    #
-            ++$c;                                                            #
-        }                                                                    #
+sub __sanitized_string {
+    my ( $self, $str ) = @_;
+    if ( ! defined $str ) {
+        $str = $self->{undef};
     }
-    $progress->update( $total ) if $show_progress && $total >= $next_update; #
+    elsif ( ref $str ) {
+        $str = $self->__handle_reference( $str );
+    }
+    elsif ( $self->{binary_filter} && substr( $str, 0, 100 ) =~ $self->{binray_regexp} ) {
+        $str = $self->{binary_string};
+    }
+    else {
+        $str =~ s/^\p{Space}+//;
+        $str =~ s/\p{Space}+\z//;
+        $str =~ s/\p{Space}+/ /g;
+        $str =~ s/\p{C}//g;
+    }
+    return $str;
 }
 
 sub __handle_reference {
@@ -455,6 +363,58 @@ sub __handle_reference {
 }
 
 
+sub __calc_col_width {
+    my ( $self, $a_ref ) = @_;
+    my $show_progress = $self->{show_progress} >= 2 ? 1 : 0; #
+    my $total = @$a_ref;                      #
+    my $next_update = 0;                      #
+    my $c = 0;                                #
+    my $progress;                             #
+    if ( $show_progress ) {                   #
+        local $| = 1;                         #
+        print CLEAR_SCREEN;                   #
+        $progress = Term::ProgressBar->new( { #
+            name => 'Computing',              #
+            count => $total,                  #
+            remove => 1 } );                  #
+        $progress->minor( 0 );                #
+    }                                         #
+    $self->{longest_col_name} = 0;
+    $self->{width_cols} = [ ( 1 ) x @{$a_ref->[0]} ];
+    my $normal_row = 0;
+    my @col_idx = ( 0 .. $#{$a_ref->[0]} );
+
+    for my $row ( @$a_ref ) {
+        for my $i ( @col_idx ) {
+            my $width = print_columns( $self->__sanitized_string( $row->[$i] ) );
+            if ( $normal_row ) {
+                if ( $width > $self->{width_cols}[$i] ) {
+                    $self->{width_cols}[$i] = $width;
+                }
+                if ( $row->[$i] && ! looks_like_number $row->[$i] ) {
+                    ++$self->{not_a_number}[$i];
+                }
+            }
+            else {
+                # col name
+                $self->{width_head}[$i] = $width;
+                if ( $width > $self->{longest_col_name} ) {
+                    $self->{longest_col_name} = $width;
+                }
+                if ( $i == $#$row ) {
+                    $normal_row = 1;
+                }
+            }
+        }
+        if ( $show_progress ) {                                              #
+            $next_update = $progress->update( $c ) if $c >= $next_update;    #
+            ++$c;                                                            #
+        }                                                                    #
+    }
+    $progress->update( $total ) if $show_progress && $total >= $next_update; #
+}
+
+
 sub __calc_avail_width {
     my ( $self, $a_ref, $term_width ) = @_;
     my $width_head = [ @{$self->{width_head}} ];
@@ -475,7 +435,7 @@ sub __calc_avail_width {
             last HEAD if $count == 0;
             $sum += $count;
         }
-        return $width_head, $width_cols;
+        return $width_cols;
     }
     elsif ( $sum > $avail_width ) {
         my $minimum_with = $self->{min_col_width} || 1;
@@ -500,7 +460,9 @@ sub __calc_avail_width {
             ++$percent;
             my $count = 0;
             for my $i ( 0 .. $#width_cols_tmp ) {
-                next if $minimum_with >= $width_cols_tmp[$i];
+                if ( $minimum_with >= $width_cols_tmp[$i] ) {
+                    next;
+                }
                 if ( $minimum_with >= _minus_x_percent( $width_cols_tmp[$i], $percent ) ) {
                     $width_cols_tmp[$i] = $minimum_with;
                 }
@@ -543,7 +505,7 @@ sub _minus_x_percent {
 
 sub __trunk_col_to_avail_width {
     my ( $self, $a_ref, $width_cols ) = @_;
-    my $total = $#{$a_ref};                   #
+    my $total = @$a_ref;                      #
     my $next_update = 0;                      #
     my $c = 0;                                #
     my $progress;                             #
@@ -561,15 +523,15 @@ sub __trunk_col_to_avail_width {
     for my $row ( @$a_ref ) {
         my $str = '';
         for my $i ( 0 .. $#$width_cols ) {
-            $str .= unicode_sprintf( $row->[$i], $width_cols->[$i], $self->{not_a_number}[$i] ? 0 : 1 );
+            $str .= unicode_sprintf(
+               $self-> __sanitized_string( $row->[$i] ),
+                $width_cols->[$i],
+                $self->{not_a_number}[$i] ? 0 : 1
+            );
             $str .= $tab if $i != $#$width_cols;
         }
         push @$list, $str;
         if ( $self->{show_progress} ) {                                      #
-            my $is_power = 0;                                                #
-            for ( my $i = 0; 2 ** $i <= $c; ++$i ) {                         #
-                $is_power = 1 if 2 ** $i == $c;                              #
-            }                                                                #
             $next_update = $progress->update( $c ) if $c >= $next_update;    #
             ++$c;                                                            #
         }                                                                    #
@@ -596,7 +558,7 @@ Term::TablePrint - Print a table to the terminal and browse it interactively.
 
 =head1 VERSION
 
-Version 0.018_01
+Version 0.026_01
 
 =cut
 
@@ -630,19 +592,8 @@ cursor reaches the topmost line, the previous page is shown automatically if it
 
 If the terminal is too narrow to print the table, the columns are adjusted to the available width automatically.
 
-=head3 print_table called in void context.
-
-If the option I<table_expand> is enabled and a row is selected with C<Return>, each column of that row is output in its
-own line preceded by the column name. This might be useful if the columns were cut due to the too low terminal width.
-
-=head3 print_table called in scalar context.
-
-If a row is selected with C<Return>, C<print_table> returns the index of that row.
-
-=head3 print_table called in list context.
-
-In list context it can be marked rows with the C<SpaceBar>. When C<Return> is pressed C<print_table> returns the indexes
-of the marked row including the highlighted row.
+If the option table_expand is enabled and a row is selected with Return, each column of that row is output in its own
+line preceded by the column name. This might be useful if the columns were cut due to the too low terminal width.
 
 To get a proper output C<print_table> uses the C<columns> method from L<Unicode::GCString> to calculate the string
 length.
@@ -723,8 +674,14 @@ row of the table.
 
 =back
 
-The C<Return> key closes the table if the cursor is on the header row. If I<keep_header> and I<table_expand> are
-enabled, the table closes by selecting the first row twice in succession.
+With I<keep_header> disabled the C<Return> key closes the table if the cursor is on the header row.
+
+If I<keep_header> is enabled and I<table_expand> is set to C<0>, the C<Return> key closes the table if the cursor is on
+the first row.
+
+If I<keep_header> and I<table_expand> are enabled and the cursor is on the first row, pressing C<Return> three times in
+succession closes the table. If I<table_expand> is set to C<1> and the cursor is auto-jumped to the first row, it is
+required only one C<Return> to close the table.
 
 If the cursor is not on the first row:
 
@@ -753,6 +710,10 @@ see option L</choose_columns>.
 
 Defaults may change in a future release.
 
+=head3 prompt
+
+String displayed above the table.
+
 =head3 add_header
 
 If I<add_header> is set to 1, C<print_table> adds a header row - the columns are numbered starting with 1.
@@ -826,8 +787,9 @@ Default: 2
 
 =head3 table_expand
 
-If the option I<table_expand> is set to 1 and C<Return> is pressed, the selected table row is printed with each column
-in its own line.
+If the option I<table_expand> is set to C<1> or C<2> and C<Return> is pressed, the selected table row is printed with
+each column in its own line. Exception: if I<table_expand> is set to C<1> and the cursor auto-jumped to the first row,
+the first row will not be expanded.
 
 If I<table_expand> is set to 0, the cursor jumps to the to first row (if not already there) when C<Return> is pressed.
 
@@ -879,7 +841,7 @@ if the first argument refers to an empty array.
 
 =head2 Perl version
 
-Requires Perl version 5.8.0 or greater.
+Requires Perl version 5.8.3 or greater.
 
 =head2 Decoded strings
 
@@ -914,7 +876,7 @@ Matthäus Kiem <cuer2s@gmail.com>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2012-2014 Matthäus Kiem.
+Copyright 2013-2016 Matthäus Kiem.
 
 This library is free software; you can redistribute it and/or modify it under the same terms as Perl 5.10.0. For
 details, see the full text of the licenses in the file LICENSE.
